<?php
/**
 * Copyright (C) 2019 Leipzig University Library
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Dorian Merz <merz@ub.uni-leipzig.de>
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:plugins:hierarchy_components Wiki
 */
namespace fid\Helper;

use DOMDocument;
use VuFind\Date\Converter;
use VuFindHttp\HttpServiceAwareInterface as HttpServiceAwareInterface;
use Laminas\Cache\Storage\StorageInterface;
use Laminas\Config\Config;
use Laminas\View\Helper\AbstractHelper;

/**
 * Class Rss Provides feed reader functionality
 * Mostly a copy from instance/fid_adlr,
 * * adds excerpt and date,
 * * returns objects instead of arrays
 * * provides __invoke
 * * alters constructor to include config and dateConverter
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Dorian Merz <merz@ub.uni-leipzig.de>
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:plugins:hierarchy_components Wiki
 */
class Rss extends AbstractHelper implements HttpServiceAwareInterface
{
    use \VuFindHttp\HttpServiceAwareTrait;

    /**
     * Config for rss
     *
     * @var Config
     */
    protected $rssConfig;

    /**
     * Date converter
     *
     * @var Converter
     */
    protected $dateConverter;

    /**
     * Cache for rss content
     *
     * @var StorageInterface
     */
    protected $cache;

    /**
     * Is Caching activated
     *
     * @var bool
     */
    protected $caching;

    /**
     * Key for cache
     *
     * @var string
     */
    protected $cacheKey;

    /**
     * Rss constructor.
     *
     * @param Config           $rssConfig     config
     * @param Converter        $dateConverter converter
     * @param StorageInterface $cache         cache manager
     */
    public function __construct(Config $rssConfig, Converter $dateConverter, StorageInterface $cache)
    {
        $this->rssConfig = $rssConfig;
        $this->dateConverter = $dateConverter;
        $this->feedDateFormat = $this->rssConfig->input_date_format ?? 'c';
        $this->prepareCache($cache);
    }

    /**
     * Prepare cache
     *
     * @param StorageInterface $cache cache
     *
     * @return void
     */
    protected function prepareCache(StorageInterface $cache)
    {
        $ttl = $this->rssConfig['cache_ttl'] ?? 0;
        if ($ttl == -1) {
            $this->caching = false;
        } else {
            $this->caching = true;
            $this->cache = $cache;
            $this->cache->getOptions()->setTtl($ttl);
            $this->cacheKey = md5(implode($this->rssConfig->toArray()));
        }
    }

    /**
     * Read cached feed
     *
     * @return bool|mixed cached feed or false if caching is inactive
     */
    protected function readCacheData()
    {
        if ($this->caching) {
            return $this->cache->getItem($this->cacheKey);
        }
        return false;
    }

    /**
     * Write data to cache
     *
     * @param $data content
     *
     * @return void
     */
    protected function writeCacheData($data)
    {
        if ($this->caching) {
            $this->cache->setItem($this->cacheKey, $data);
        }
    }

    /**
     * Read the feed
     *
     * @return array|bool|mixed
     */
    public function __invoke()
    {
        return $this->getFeed();
    }

    /**
     * Get Rss feed data
     *
     * @return array|bool|mixed
     */
    protected function getFeed()
    {
        if ($cacheData = $this->readCacheData()) {
            return $cacheData;
        } else {
            $feed = $this->load($this->rssConfig);
            $this->writeCacheData($feed);
            return $feed;
        }
    }

    /**
     * Get rss data
     *
     * @param $rssOrigin array of rss config data
     *
     * @return array of rss feeds
     */
    public function load($rssOrigin)
    {
        $baseUrl    = $rssOrigin["base_url"] ?? "";
        $fallback   = $rssOrigin["fallback_url"] ?? "";
        $docPath    = $rssOrigin["xml_doc"] ?? "";

        $feeds = [];
        try {
            $rss = $this->httpService->get($baseUrl . '/' . $docPath);
            /* var DOMDocument $rssDoc */
            $rssDoc = new \DOMDocument();
            if ($rss->isSuccess() && $rssDoc->loadXML($rss->getBody())) {
                $limit = $rssOrigin['limit'] ?? 3;
                $n = 0;

                foreach ($rssDoc->getElementsByTagName('item') as $node) {
                    $excerpt = $node->getElementsByTagName('description')->item(0)->nodeValue;
                    if (strlen($excerpt) > $this->rssConfig->excerpt_length) {
                        $excerpt = substr($excerpt, 0, $this->rssConfig->excerpt_length);
                        $lastSpace = strrpos($excerpt, ' ');
                        $excerpt = substr($excerpt, 0, $lastSpace);
                        if (isset($this->rssConfig['excerpt_suffix'])) {
                            $excerpt .= $this->rssConfig['excerpt_suffix'];
                        }
                    }
                    $date = $node->getElementsByTagName('pubDate')->item(0)->nodeValue;
                    $date = $this->dateConverter->convertToDisplayDate($this->feedDateFormat, $date);
                    $item = [
                        'title' => $node->getElementsByTagName('title')->item(0)->nodeValue,
                        'link' => $node->getElementsByTagName('link')->item(0)->nodeValue,
                        'excerpt' => $excerpt,
                        'date' => $date,
                    ];
                    array_push($feeds, (object)$item);

                    if (++$n >= $limit) {
                        break;
                    }
                }
            }
        } catch (\Exception $ex) {
            if ($fallback) {
                $item = [
                    'title' => $fallback,
                    'link' => $fallback
                ];
                array_push($feeds, $item);
            }
        }

        return $feeds;
    }
}
