<?php
/**
 * Get It box view helper
 *
 * PHP version 7
 *
 * Copyright (C) Leipzig University Library 2019.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\View\Helper\Root;

use VuFind\I18n\Translator\TranslatorAwareTrait;
use VuFind\RecordDriver\AbstractBase;
use VuFind\Role\PermissionManager;
use Laminas\View\Helper\AbstractHelper;

/**
 * Get It box view helper
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
class GetIt extends AbstractHelper
{
    use TranslatorAwareTrait;

    /* @var AbstractBase */
    protected $driver;

    /**
     * FID API Client
     *
     * @var \fid\Service\Client
     */
    protected $fidClient;

    /**
     * Permission Manager
     *
     * @var PermissionManager
     */
    protected $permissionManager;

    /**
     * Solr field source_id
     *
     * @var
     */
    protected $sid;

    /**
     * Solr field format
     * or instance specific configuration
     *
     * @var
     */
    protected $format;

    /**
     * Solr field facet_avail
     *
     * @var
     */
    protected $facetAvail;

    /**
     * Solr field multipart_set
     *
     * @var
     */
    protected $multipart;

    /**
     * Has a hierarchy parent title
     *
     * @var
     */
    public $isCollection;

    /**
     * Solr field institution
     *
     * @var
     */
    protected $institution;

    /**
     * Solr field mega_collection
     *
     * @var
     */
    protected $megacollection;

    /**
     * Solr field collection
     *
     * @var
     */
    protected $collection;

    /**
     * Solr field isbn
     *
     * @var
     */
    protected $isbn;

    /**
     * Solr field issn
     *
     * @var
     */
    protected $issn;

    /**
     * Rules to be evaluated
     *
     * @var Config rules config
     */
    protected $getItRules;

    /**
     * GetIt constructor.
     *
     * @param $fidClient         FIDIS Client
     * @param $permissionManager Permission Manager
     * @param $getItRules        Config Manager
     */
    public function __construct($fidClient, $permissionManager, $getItRules)
    {
        $this->fidClient = $fidClient;
        $this->permissionManager = $permissionManager;

        if (!empty($getItRules)) {
            if (isset($getItRules['rules']) && isset($getItRules['stopFlags'])) {
                $this->getItRules = $getItRules;
            }
        }
    }

    /**
     * Get Solr field data
     *
     * @param $driver Record Driver
     *
     * @return GetIt GetIt View Helper
     */
    public function __invoke($driver)
    {
        $this->driver = $driver;
        $this->sid = $this->driver->tryMethod('getSourceID');
        $this->format = $this->driver->tryMethod('getFormats');
        $this->facetAvail = $this->driver->tryMethod('getFacetAvail');
        //$this->multipart = $this->driver->tryMethod('isMultiPartSet');
        $this->isCollection = $this->driver->tryMethod('hasChildren');
        $this->institution = $this->driver->tryMethod('getInstitutions');
        $this->megacollection = $this->driver->tryMethod('getMegaCollection');
        $this->collection = $this->driver->tryMethod('getCollection');
        $this->isbn = $this->driver->tryMethod('getISBNs');
        $this->issn = $this->driver->tryMethod('getISSNs');

        return $this;
    }

    /**
     * Check if User is authorized
     *
     * @param $permission Permission
     *
     * @return bool
     */
    public function hasPermission($permission)
    {
        return $this->permissionManager->isAuthorized($permission);
    }

    /**
     * This function exists so it's easier to adjust settings within the rules evaluator engine
     *
     * @return array context for rules evaluator
     */
    protected function getContext(): array
    {
        return [
            'this' => $this,
            'client' => $this->fidClient,
            'permissionmanager' => $this->permissionManager,
            'driver' => $this->driver,
            'sid' => $this->sid,
            'format' => $this->format,
            'facet_avail' => $this->facetAvail,
            'is_collection' => $this->isCollection,
            'institution' => $this->institution,
            'megacollection' => $this->megacollection,
            'collection' => $this->collection,
            'isbn' => $this->isbn,
            'issn' => $this->issn,
            'hasEBookAccess' => $this->hasPermission('fid.EBooks')
        ];
    }

    /**
     * Read specific config of given record
     * Set default Values of getit box
     *
     * @return array config of given record
     */
    public function getConfig()
    {
        $eval = new \finc\Rules\Evaluator\Evaluator(
            array_values($this->getItRules['rules']),
            (array)$this->getItRules['stopFlags']
        );
        $context = $this->getContext();
        // set variables of config in context
        foreach ($this->getItRules as $key => $value) {
            if ($key != "rules" and $key != "stopFlags") {
                $context[$key] = $value;
            }
        }
        return $eval($context);
    }

    /**
     * Helper function to check simple intersect
     *
     * @param array $a first array
     * @param array $b second array
     *
     * @return bool
     */
    public function isArrayIntersect($a, $b)
    {
        return !empty(array_intersect($a, $b));
    }

    /**
     * Helper function to promt notices in short syntax
     *
     * @param string $key translation key suffix after "getit_text_"
     *
     * @return string
     */
    public function out($key)
    {
        if ($key == "multipart_mono") {
            return $this->translate(
                'getit_text_multipart_mono',
                ['%%hierarchy_tree%%' => $this->translate('hierarchy_tree')]
            );
        } else {
            return $this->translate("getit_text_{$key}");
        }
    }

    /**
     * Has User ordered certain Record (SID 51)
     *
     * @return bool
     */
    public function isRecordOrdered()
    {
        if ($this->fidClient->isLoggedOn()) {
            $id = $this->driver->tryMethod('getUniqueID');
            $record = $this->fidClient->requestRecord($id);

            return $record->getOrdered();
        }
        return false;
    }
}
